using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Labs.Interfaces;
using Labs.Business;

namespace Labs.Tests
{
    [TestClass]
    public class CustomerManagerFixture
    {
        [TestMethod]
        public void CreateSuggestedCustomerAccountCode()
        {
            var manager = new CustomerManager(
                new MockCustomerRepository());

            string result = manager.GetSuggestedAccountCode();

            Assert.IsFalse(String.IsNullOrWhiteSpace(result),
                "Result should not be null or whitespace.");

            Assert.AreEqual<int>(6, result.Trim().Length,
                "Account code should be 6 characters long.");
        }

        [TestMethod]
        public void CreateAccountCodeSucceedsWhenCodeIsUnique()
        {
            var mockRepository = new MockCustomerRepository();

            var manager = new CustomerManager(mockRepository);

            string result = manager.CreateAccountCode("Customer 123");

            Assert.IsFalse(String.IsNullOrWhiteSpace(result),
                "Result should not be null or whitespace.");

            mockRepository.AssertSaveCustomerWasCalled();
            mockRepository.AssertIsAccountCodeInUseWasCalled();
        }

        [TestMethod]
        public void CreateAccountCodeRetriesWhenOriginalIsInUse()
        {
            RunAndAssertIsAccountCodeInUseForNumberOfAttempts(2);
        }

        [TestMethod]
        public void CreateAccountCodeRetriesUntilUniqueCodeIsFound()
        {
            RunAndAssertIsAccountCodeInUseForNumberOfAttempts(10);
        }

        private void RunAndAssertIsAccountCodeInUseForNumberOfAttempts(
            int attemptCount)
        {
            var mockRepository = new MockCustomerRepository();

            // number of attempts required to get a unique code
            mockRepository.NumberOfRequiredIsAccountCodeInUseAttempts = attemptCount;

            var manager = new CustomerManager(mockRepository);

            string result = manager.CreateAccountCode("Customer 123");

            Assert.IsFalse(String.IsNullOrWhiteSpace(result),
                "Result should not be null or whitespace.");

            mockRepository.AssertSaveCustomerWasCalled();

            // check that it was called twice
            mockRepository.AssertIsAccountCodeInUseWasCalled(attemptCount);

            mockRepository.AssertReceivedAccountCodesWereDifferent();
        }

                
    }
}